# Copyright (c) 2012: Thiago Chaves de Oliveira Horta <shundread@gmail.com>
#
# Authors: Thiago Chaves de Oliveira Horta
#
# License: GPL v3 (see gpl-3.0.txt)

import pygame

from constant import *

# Resources
images = {}
sprites = {}
tilesets = {}

# Classes
class Animation(object):
    def __init__(self, frames, frame_duration):
        self.frames = frames
        self.frame_duration = frame_duration
        self.length = frame_duration * len(frames)
        self.timer = 0

    def update(self, dt):
        self.timer = (self.timer + dt) % self.length

    def reset(self):
        self.timer = 0

    def get_frame(self):
        index = int(self.timer // self.frame_duration)
        return self.frames[index]

class Sprite(object):
    def get_frame(self):
        pass

    def get_size(self):
        return self.get_frame().get_size()

    def draw(self, surface, position, shift=(0, 0)):
        pos = (position[0] + shift[0], position[1] + shift[1])
        surface.blit(self.get_frame(), position)

    def update(self, dt):
        pass

    def set_animation(self, animation_id):
        pass

class StaticSprite(Sprite):
    def __init__(self, image):
        Sprite.__init__(self)
        self.image = image

    # overloads
    def get_frame(self):
        return self.image

class AnimatedSprite(Sprite):
    def __init__(self, animations):
        Sprite.__init__(self)
        self.animations = animations
        self.current_animation_id = 0
        self.current_animation = self.animations[0]

    def update(self, dt):
        self.current_animation.update(dt)

    def get_frame(self):
        return self.current_animation.get_frame()

    def set_animation(self, animation_id):
        if animation_id != self.current_animation_id:
            self.current_animation = self.animations[animation_id]
            self.current_animation_id = animation_id
            self.current_animation.reset()

# Initialization
def init(resolution=DefaultResolution):
    pygame.display.set_mode(resolution)

# Image conversion and splitting
def convert(surface, alpha=UseAlpha):
    if alpha:
        return surface.convert_alpha()
    else:
        result = surface.convert()
        result.set_colorkey(ColorKey)
        return result

def split(surface, (w, h)):
    chunks = []
    for y in range(0, surface.get_height(), h):
        for x in range(0, surface.get_width(), w):
            chunks.append(surface.subsurface((x, y, w, h)))
    return chunks

def tiles(surface):
    chunks = []
    for y in range(0, surface.get_height(), TileSize):
        for x in range(0, surface.get_width(), TileSize):
            chunks.append(surface.subsurface((x, y, TileSize, TileSize)))
    return chunks

# Loading files
def load_image(path, alpha=True):
    if not images.has_key(path, alpha):
        images[path] = convert(pygame.image.load(path),  alpha)
    return images[path]

def load_sprite(path, size=(XScale, YScale), alpha=True):
    if not sprites.has_key(path):
        sprites[path] = split(convert(pygame.image.load(path), alpha), size)
    return sprites[path]

def load_tileset(path, alpha=False):
    if not tilesets.has_key(path):
        tilesets[path] = tiles(convert(pygame.image.load(path), alpha))
    return tilesets[path]

# Animation map making
def default_animation_map(frames):
    animation_map = {
        Move|Down  : Animation(frames[0:3]  + [frames[1]],  200),
        Move|Left  : Animation(frames[3:6]  + [frames[4]],  200),
        Move|Right : Animation(frames[6:9]  + [frames[7]],  200),
        Move|Up    : Animation(frames[9:12] + [frames[10]], 200),
        Wait|Down  : Animation([frames[1]],  200),
        Wait|Left  : Animation([frames[4]],  200),
        Wait|Right : Animation([frames[7]],  200),
        Wait|Up    : Animation([frames[10]], 200)
    }
    return animation_map

