# Copyright (c) 2012: Thiago Chaves de Oliveira Horta <shundread@gmail.com>
#
# Authors: Thiago Chaves de Oliveira Horta
#
# License: GPL v3 (see gpl-3.0.txt)

from os.path import join

import pygame

from actor import *
from entity import *
from graphics import *
from constant import *
from utils import *

# Classes
class Tile(object):
    def __init__(self, image, solid=False):
        self.image = image
        self.solid = solid

    def is_solid(self):
        return self.solid

    def draw(self, surface, position, shift=(0, 0)):
        surface.blit(self.image, position)
        #self.image.draw(surface, position, shift)

class Layer(object):
    def __init__(self, name, tiles, active=True, visible=True):
        self.name = name
        self.tiles = tiles
        self.active = active
        self.visible = visible
        self.size = (len(tiles), len(tiles[0]))

    def at(self, (x_index, y_index)):
        (i, j) = (y_index, x_index)
        return self.tiles[i][j]

    def get_size(self):
        return (self.size[1], self.size[0])

    def is_active(self):
        return self.active

    def is_visible(self):
        return self.visible

class Level(object):
    def __init__(self, layers, entities, actors, timers):
        self.layers = layers
        self.entities = entities
        self.actors = actors
        self.timers = timers
        self.size = layers[0].get_size()

    def get_boundaries(self):
        (w, h) = map_to_screen(self.size)
        return pygame.rect.Rect(0, 0, w, h)

    def draw(self, surface, rect):
        (xstart, ystart) = rect.topleft
        (xend, yend) = rect.bottomright
        (xcenter, ycenter) = rect.center
        (xshift, yshift) = (-xcenter, -ycenter) # Passed to entities and actors
        for layer in self.layers:
            if layer.is_visible():
                for y in range(ystart, yend, TileSize):
                    for x in range(xstart, xend, TileSize):
                        (x_index, y_index) = screen_to_map((x, y))
                        layer.at((x_index, y_index)).draw(surface, (x, y))

    def solid_at(self, (x_index, y_index)):
        # Check boundaries
        if x_index < 0: return True
        if y_index < 0: return True
        if x_index >= self.size[0]: return True
        if y_index >= self.size[1]: return True

        for layer in self.layers:
            if layer.is_active():
                if layer.at((x_index, y_index)).is_solid():
                    return True
        return False
    
    def highlight_around(self, surface, rect, target_rect):
        (xstart, ystart) = rect.topleft
        (xend, yend) = rect.bottomright
        (xcenter, ycenter) = rect.center
        (xshift, yshift) = (-rect.left, -rect.top) # Passed to entities and actors
        size = (TileSize, TileSize)
        for y in range(target_rect.top, target_rect.bottom, TileSize-1):
            for x in range(target_rect.left, target_rect.right, TileSize-1):
                pos = map_to_screen(screen_to_map((x, y)))
                drect = pygame.rect.Rect(pos, size)
                pygame.draw.rect(surface, (255, 255, 255), drect)

    def solid_around(self, rect):
        for y in range(rect.top, rect.bottom, TileSize-1):
            for x in range(rect.left, rect.right, TileSize-1):
                if self.solid_at(screen_to_map((x, y))):
                    return True
        return False

    def update(self, game_data, dt):
        # Perform thinking cycle
        for actor in self.actors: actor.think(game_data, dt)

        # Update stuff
        for timer in self.timers: timer.update(game_data, dt)
        for actor in self.actors: actor.update(game_data, dt)
        for ent in self.entities: ent.update(game_data, dt)

        # Clear lists of finished/dead stuff
        self.timers = [timer for timer in self.timers if not timer.finished()]
        self.actors = [actor for actor in self.actors if not actor.is_dead()]
        self.entities = [ent for ent in self.entities if not ent.must_destroy()]

class Level1(Level):
    def __init__(self):
        layers = load_file(join('data','testmap.map'))
        Level.__init__(self, layers, [], [], [])

# Loading files

def load_file(filepath):
    f_in = open(filepath)

    # Load map metadata
    name = f_in.readline().split(':')[-1].strip()
    width = int(f_in.readline().split(':')[-1])
    height = int(f_in.readline().split(':')[-1])
    n_layers = int(f_in.readline().split(':')[-1])

    # Load layers
    layers = []
    for l in range(n_layers):
        # Load layer metadata
        layer_name = f_in.readline().split(':')[-1].strip()
        tileset_path = join('data', f_in.readline().split(':')[-1].strip())
        tileset = load_tileset(tileset_path)
        solid = bool(int(f_in.readline().split(':')[-1]))
        visible = bool(int(f_in.readline().split(':')[-1]))

        # Load layer
        tiles = []
        for i in range(height):
            ids = [int(i)-1 for i in f_in.readline().split(',')[:width]]
            tiles.append([Tile(tileset[i], i>0) for i in ids])
        layers.append(Layer(layer_name, tiles, solid, visible))
    
    return layers

